
;  Torus - a simple 3D wireframe rotating object
;  by Matthew Shepcar of Icarus Productions


#include asm86.h
#include ti86asm.inc

_divHLbyA       =4048h

.org _asm_exec_ram

        call _runindicoff
        call BuildTrigTables
        call MakeTorus

        ld a,123
        ld bc,4567h
RotLoop:

        push af
        push bc
        call CalcTransformationMatrix

        ld hl,(ScrAddr)   ;here two frames are OR'ed onto the screen
        ld a,h            ;and one of them is also cleared simultaneously
        xor 4             ;ready for the next frame to be drawn
        ld b,a
        ld c,l
        ld de,0FC00h
BlurScreen:
        ld a,(bc)
        or (hl)
        ld (hl),0
        ld (de),a
        inc hl
        inc de
        inc bc
        ld a,d
        or a
        jr nz,BlurScreen

        ld hl,Points
        ld c,(hl)
        inc hl
        ld b,(hl)
        inc hl
        ld de,TransformedPoints
DoTransformPoints:
        push bc
        push de
        ld a,(hl)
        inc hl
        ld b,(hl)
        inc hl
        ld c,(hl)
        inc hl
        push hl
        call RotatePoint    ;each point is rotated
        push hl
        push de
        ld hl,128*128
        ld a,b
        sra a
        add a,112
        call _divHLbyA      ;and projected (x'=x/z, y'=y/z)
        ld a,l
        pop de
        push af
        call MultAxDE
        pop af
        ld c,h
        pop de
        call MultAxDE
        ld e,070h       ;squash in Y (86's pixels aren't square)
        call MultExH
        ld a,h
        pop hl
        pop de
        add a,32
        and 63
        ld (de),a
        inc de
        ld a,c
        add a,64
        and 127
        ld (de),a
        inc de
        pop bc
        dec bc
        ld a,b
        or c
        jr nz,DoTransformPoints

        ld hl,Lines
        ld c,(hl)
        inc hl
        ld b,(hl)
        inc hl
DrawLines:
        push bc
        ld e,(hl)
        inc hl
        push hl
        ld l,(hl)
        ld h,0
        ld d,h
        ld bc,TransformedPoints
        add hl,hl
        add hl,bc
        call LD_HL_MHL
        ex de,hl
        add hl,hl
        add hl,bc
        call LD_HL_MHL
        call Line       ;Lines drawn between transformed points
        pop hl
        pop bc
        inc hl
        dec bc
        ld a,b
        or c
        jr nz,DrawLines

        ld a,(ScrAddr+1)
        xor 4
        ld (ScrAddr+1),a    ;flip address of screen buffer

        ld a,0BFh
        out (1),a
        in a,(1)
        ld e,a
        pop bc
        pop af
        add a,4             ;adjust rotation angles
        dec b
        inc c
        inc c
        inc c
        bit 6,e             ;check if exit hit
        jp nz,RotLoop

      
        call _clrScrn
        jp _homeup

MakeTorus:
        ld ix,Points
        ld (ix),72
        inc ix
        ld (ix),0
        inc ix
        xor a
DoRings:
        push af
        ld h,SinCosTable/256
        ld l,a
        ld a,(hl)
        ld (TorusSin),a
        ld de,64
        add hl,de
        ld a,(hl)
        ld (TorusCos),a
        xor a
DoCircles:
        push af
        ld h,SinCosTable/256
        ld l,a
        ld a,(hl)
        sra a
        sra a
        sra a
        sra a
        ld (ix),a
        inc ix
        ld de,64
        add hl,de
        ld a,(hl)
        sra a
        sra a
        sra a
        sra a
;        sra a
        add a,25
        ld b,a
        ld h,0
TorusSin =$-1
        call MultAxH
        ld (ix),h
        inc ix
        ld a,b
        ld h,0
TorusCos =$-1
        call MultAxH
        ld (ix),h
        inc ix
        pop af
        add a,43
        jr nc,DoCircles
        pop af
        add a,22
        jr nc,DoRings

        ld hl,Lines
        ld (hl),144
        inc hl
        ld (hl),0
        inc hl
        xor a
StoreRings:
        ld b,a
        xor a
StoreCircles:
        add a,b
        ld c,a
        ld (hl),a
        inc hl
        add a,6
        cp 72
        jr c,NotLastCircle
        sub 72
NotLastCircle:
        ld (hl),a
        inc hl
        ld a,c
        ld (hl),a
        sub b
        inc hl
        inc a
        cp 6
        jr c,NotLastCircPoint
        xor a
NotLastCircPoint:
        ld c,a
        add a,b
        ld (hl),a
        ld a,c
        inc hl
        or a
        jr nz,StoreCircles
        ld a,b
        add a,6
        cp 72
        jr nz,StoreRings
        ret

FindPixel:
        ld a,b
        and 7
        or ___offsets_table & 255
        ld e,a
        ld d,___offsets_table/256
        ld h,0
        ld a,c
        add a,a
        add a,a
        rl h
        add a,a
        rl h
        add a,a
        rl h
        srl b
        srl b
        srl b
        or b
        ld l,a
        ld a,(de)
        ret

.org (($-1) | 7)+1

___offsets_table:
        .db      128,64,32,16,8,4,2,1

#if 0
CS =20
Points  .dw 8
        .db -CS,-CS,-CS
        .db -CS,-CS,CS
        .db -CS,CS,CS
        .db -CS,CS,-CS
        .db CS,-CS,-CS
        .db CS,-CS,CS
        .db CS,CS,CS
        .db CS,CS,-CS

Lines   .dw 12
        .db 0,1,1,2,2,3,3,0
        .db 4,5,5,6,6,7,7,4
        .db 0,4,1,5,2,6,3,7
#endif

ScrAddr .dw 0F000h
ScrAddr2 .dw 0

RotWS  =9000h
CamWS  =9100h
PrtWS  =9180h
Points =9800h
Lines  =0A000h
TransformedPoints =0A800h

#include 3drot.asm
#include line.asm
#include mult.asm
#include trigtab.asm

SinCosTable =(($-1) | 255)+1        ;320 bytes


.end
